﻿{

	// pt_AEtoC4D v1.4
	// Copyright (c) 2007-2010 Paul Tuersley. All rights reserved.
	// email: paul.tuersley@btinternet.com
	// Updates and other scripts available at www.aenhancers.com
	//
	// This set of scripts can be used to export data from AE to C4D.
	// They require After Effects 7.0 and Cinema 4D R9.6, or greater.
	//
	// AEtoC4D exports camera position and rotation, nulls with the position and
	// rotation of other layers, and the comp frame rate, duration and start frame.
	//
	// Thanks to Paul Babb, Rick Barrett and Liam Stacy at Maxon, Ko Maruyama,
	// Stu Maschwitz, Dale Bradshaw, Nab, Steve Kirby and Salvador Zalvidea
	// for their help and ideas.
	//
	//
	// Instructions:
	// 1. Put pt_AEtoC4D.jsx in After Effects/Scripts (MAC) or After Effects/Support Files/Scripts (WIN)
	//
	// 2. Put pt_AEtoC4D.CSC in Cinema 4D / library / scripts.
	//
	// 3. Animate your scene in After Effects using a Camera with Layer >
	// Transform > Auto-Orient set to Off. Animate rotation using the XYZ
	// Rotation properties, not Orientation.
	//
	// 4. Select the cameras and layers you want to export and choose File >
	// Scripts > pt_AEtoC4D.jsx. You'll be prompted to save a text file.
	//
	// 5. In C4D, select Plugins > User Scripts > pt_AEtoC4D and select
	// that text file.
	//
	// NOTES:
	// 1. This script doesn't export camera zoom. You'll have to set the
	// camera's field of view manually in C4D after import. Check out the 
	// test project, which uses nulls as a guide to help line up the FOV.
	//
	// 2. This script doesn't export parented layers.


	var posProp, xVal, yVal, zVal, hasKeys;
	var rotProp = new Array();
	var hadParents = false;
	
	if (parseFloat(app.version) < 7) {	// check AE is 7.0+
		alert("This script requires After Effects 7.0 or greater.");
	} else {
		// check that a comp is active and layers are selected
		var theComp = app.project.activeItem;
		if (!(theComp instanceof CompItem) || theComp.selectedLayers.length == 0) {
			alert("AE to C4D\rSelect the layers you want to export before running this script.");
		} else {
			// prompt user to save file
			//var theFileName = theComp.name + "_" + "AEtoC4D.txt"; 		
			//var theFile = filePutDialog("Save the AEtoC4D export file.", theFileName, "TEXT txt"); 
			
			// v1.4 added CS5 compatibility
			var theFile = new File (Folder.myDocuments.absoluteURI + "/" + theComp.name + "_" + "AEtoC4D.txt");
			theFile = theFile.saveDlg("Save the AEtoC4D export file.");

			if (theFile != null) {	// check user didn't cancel dialog
			
			theFile.lineFeed = "macintosh";
			theFile.open("w","TEXT","????");

				// write the comp settings
				var theStartFrame = app.project.displayStartFrame + (theComp.displayStartTime / theComp.frameDuration);
				var numFrames = Math.floor(theComp.duration / theComp.frameDuration); 				
				theFile.writeln("FrameRate," + (1 / theComp.frameDuration));
				theFile.writeln("StartFrame," + (theStartFrame));
				theFile.writeln("Duration," + numFrames);
				theFile.writeln();
	
				// loop through the selected layers, bottom to top
				for (var b = (theComp.selectedLayers.length -1); b >= 0; b--) {		
					curLayer = theComp.selectedLayers[b];
					
					// ignore parented layers
					if (curLayer.parent != null) {
						hadParents = true;
					} else {
						// write Camera or Null header
						if (curLayer instanceof CameraLayer) {
							theFile.writeln("Camera," + curLayer.name);
						} else {
							theFile.writeln("Null," + curLayer.name);
						}

						posProp = curLayer.property("ADBE Transform Group").property("ADBE Position");
						rotProp[0] = curLayer.property("ADBE Transform Group").property("ADBE Rotate X");
						rotProp[1] = curLayer.property("ADBE Transform Group").property("ADBE Rotate Y");
						rotProp[2] = curLayer.property("ADBE Transform Group").property("ADBE Rotate Z");

						// save position data;		
						// static value if no keys or expression, otherwise save baked keyframes	
						if (posProp.numKeys == 0 && posProp.expressionEnabled == false) {
							theFile.writeln("Position," + posProp.value[0] + "," + posProp.value[1] + "," + posProp.value[2]);	
						} else {	
							theFile.writeln("Position"); 					
							for (var a = 0; a < numFrames; a++) {
								xVal = posProp.valueAtTime(a * theComp.frameDuration, false)[0]; 
								yVal = posProp.valueAtTime(a * theComp.frameDuration, false)[1];
								zVal = posProp.valueAtTime(a * theComp.frameDuration, false)[2]; 
								theFile.writeln((a + theStartFrame) + "," + xVal + "," + yVal + "," + zVal);
							}
						}
						// save rotation data;
						// check the three rotation properties for keys or expressions															
						hasKeys = false;
						for (var a = 0; a < 3; a++){						
							if (rotProp[a].numKeys != 0 || rotProp[a].expressionEnabled == true) {
								hasKeys = true;
								break;
							}
						}
						// static value if no keys or expression, otherwise save baked keyframes		
						if (hasKeys == false) {
							theFile.writeln("Rotation," + rotProp[0].value + "," + rotProp[1].value + "," + rotProp[2].value);
							theFile.writeln();	
						} else {
							theFile.writeln("Rotation");
						
							for (a = 0; a < numFrames; a++) { 
								xVal = rotProp[0].valueAtTime(a * theComp.frameDuration, false);
								yVal = rotProp[1].valueAtTime(a * theComp.frameDuration, false); 
								zVal = rotProp[2].valueAtTime(a * theComp.frameDuration, false); 
								theFile.writeln((a + theStartFrame) + "," + xVal + "," + yVal + "," + zVal);
							}
							theFile.writeln();
						}	
					}
				}
				theFile.close();
				if (hadParents == true) {
					alert("AE to C4D\rAE to C4D doesn't export parented layers. All other layers have now been exported successfully.");
				}
			}
		} 
	}
} 

